const constants = require('../constants');
const TrackviaAPI = require('trackvia-api');
const moment = require('moment');
const sharp = require('sharp');
const fs = require('fs');

const api = new TrackviaAPI(constants.key, constants.token, constants.host, constants.accountId);

module.exports = async (viewId, record, schema) => {
    let formattedRecord = {};
    await Promise.allSettled(Object.keys(record).map(async field => {
        // Field should not contain any whitespace or special characters
        const formattedField = field.replace(/\s+/g, '_').replace(/#|\"|\'|!|@|\$|%|\^|\(|\)|\*|=|\+|&*/g,'');
        const meta = schema.find(metaField => metaField.name === field);
        if(meta) {
            switch(meta.type) {
                case 'date':
                    formattedRecord[formattedField] = moment(record[field]).format('MM/DD/YYYY');
                    break;
                case 'datetime':
                    formattedRecord[formattedField] = moment(record[field]).format('MM/DD/YYYY h:mm:ss a');
                    break;
                case 'currency':
                    formattedRecord[formattedField] = `$${record[field].toString().replace(/\B(?=(\d{3})+(?!\d))/g, ',')}`;
                    break;
                case 'percentage':
                    formattedRecord[formattedField] = `${(parseFloat(record[field])*100)}%`;
                    break;
                case 'image':
                    let image;
                    try {
                        // API returns 404 if field is empty
                        image = await api.getFile(viewId, record.id, field);
                    } catch (err) {
                        // In case of an image being absent on the record
                        if(err.statusCode !== 404) {
                            console.log('Error fetching image');
                        }
                        const imageB64 = `data:image/${type};base64,${Buffer.from(fs.readFileSync('../assets/noimageavailable.png'), 'binary').toString('base64')}`;
                        formattedRecord[formattedField] = imageB64;
                        break;
                    }
                    // Extract the image type from the reponse header
                    const fileName = image.response.headers['content-disposition'].match(/"(.*)"/).pop();
                    let type = fileName.split('.').pop().toLowerCase();
                    if(type === 'jpg') {
                        type = 'jpeg';
                    }
                    // Resize the file
                    const filePath = `/tmp/${Date.now()}`;
                    try {
                        fs.writeFileSync(filePath, image.response.body, 'binary');
                        await sharp(filePath).resize(constants.imageWidth, constants.imageHeight).toFile(`${filePath}.tmp`);
                        // Convert file contents to base64 string for transport
                        const imageB64 = `data:image/${type};base64,${Buffer.from(fs.readFileSync(`${filePath}.tmp`), 'binary').toString('base64')}`;
                        // Finally attach converted image to record
                        formattedRecord[formattedField] = imageB64;
                        fs.unlinkSync(filePath);
                        fs.unlinkSync(`${filePath}.tmp`);
                    } catch (err) {
                        console.log('Could not resize file');
                    }
                    break;
                default:
                    formattedRecord[formattedField] = record[field];
                    break;
            }
        }
    }));

    return formattedRecord;
}
